#include <math.h>
#include "box.h"

//*********************************************************************
//******************       CONSTRUCTEURS         **********************
//*********************************************************************
box::box ()
{	dim=2;
	data=new interval[2];
        for (int i=0;i<=1;i++) data[i].isInterval=true;
	isEmpty=false;
        pere=-1;fg=-1;fd=-1;number=-1;critere=-1;
        f=interval(0,9999);
        F=interval(0,9999);
        deja_evalue=false;
        nombre_ascendants=0;
}
//----------------------------------------------------------------------
box::box (int dim1)
{	dim=dim1;
	data=new interval[dim1];
        for (int i=0;i<=dim1-1;i++) data[i].isInterval=true;
	isEmpty=false;
        pere=-1;fg=-1;fd=-1;number=-1;critere=-1;
        f=interval(0,9999);
        F=interval(0,9999);
        deja_evalue=false;
        nombre_ascendants=0;
}
//----------------------------------------------------------------------
box::box (const box &v)
{	if (&v == this) return;
	dim=v.dim;
	isEmpty=v.isEmpty;
	data=new interval[dim];
	for (int k=1; k<=dim; k++) (*this)[k]=v[k];
        pere=v.pere;
        fg=v.fg;
        fd=v.fd;
        number=v.number;
        f=v.f;
        F=v.F;
        critere=v.critere;
        deja_evalue=v.deja_evalue;
        nombre_ascendants=v.nombre_ascendants;
}

//----------------------------------------------------------------------
void box::operator= (const box &v)
{	if (this == &v) return;
	delete [] data;
	dim=v.dim;
	isEmpty=v.isEmpty;
	data=new interval[dim];
	for (int k=1; k<=dim; k++) (*this)[k]=v[k];
        pere=v.pere;
        fg=v.fg;
        fd=v.fd;
        number=v.number;
        f=v.f;
        F=v.F;
        critere=v.critere;
        deja_evalue=v.deja_evalue;
        nombre_ascendants=v.nombre_ascendants;
}



//----------------------------------------------------------------------
box::~box () {delete [] data;}
//----------------------------------------------------------------------
//*********************************************************************
//******************    FUNCTIONS MEMBRES     *************************
//*********************************************************************
interval& box::operator[] (int i)  const
	{		return data[i-1];
	}
//----------------------------------------------------------------------
void box::Resize (int dim1)
{	box a(dim1);
	for (int k=1; (k<=dim1)&&(k<=dim); k++)  {a[k]=(*this)[k];}
	delete [] data;
    isEmpty=false;
	data=new interval[dim1];
    for (int k=1; (k<=dim1)&&(k<=dim); k++)  {(*this)[k]=a[k];}
    dim=dim1;
}
//------------------------------------------------------
box box::centre_gravite()
{
box result;
double c;
c = 0.5*(Inf((*this)[1])+Sup((*this)[1]));
result[1] = interval(c,c);
c = 0.5*(Inf((*this)[2])+Sup((*this)[2]));
result[2] = interval(c,c);
return result;
}
//
double box::width()
{
double temp=0;
for (int k=1;k < dim+1;k++)
        {
        interval I = (*this)[k];
        if ((Width(I)) > temp) {temp = Width(I);}
        }
return temp;
}
//
double box::min_width()
{
double temp=Width((*this)[1]);
for (int k=2;k < dim+1;k++)
        {
        interval I = (*this)[k];
        if ((Width(I)) < temp) {temp = Width(I);}
        }
return temp;
}


//----------------------------------------------------------------------
box operator+(const box& v,const box& w)
	{	box r(v.dim);
		for (int k=1; k<=r.dim; k++) r[k]=v[k]+w[k];
		return (r);
	}
//----------------------------------------------------------------------
box operator-(const box& v)
	{
		box r(v.dim);
		for (int k=1; k<=r.dim; k++) r[k]=-v[k];
		return (r);
	}
//----------------------------------------------------------------------
box operator-(const box& v,const box& w) {return (v+(-w)); }
//----------------------------------------------------------------------
box operator*(const interval& a,const box& v)
	{	box r(v.dim);
		for (int k=1; k<=r.dim; k++) r[k]=a*v[k];
		return (r);
	}
//----------------------------------------------------------------------
box	operator*(const box& v,const interval& a)
	{return (a*v);}
//----------------------------------------------------------------------
ostream& operator<<(ostream& os, const box& v)
{   cout << "Box \n";
	cout << "   dim="<<v.dim <<'\n';
    if (v.isEmpty) os<<"EmptyBox";
	for (int i=1; i<=v.dim; i++)
		os << "  " <<i<< ": "<<v[i]<< "\n";
	return (os);
}
//---------------------------------------------------------------------

bool operator<(const box &x, const box &y)
 {
     return x.critere < y.critere;
 }

//---------------------------------------------------------------------
/*double box::aire()
{
double tmp = 1;
       for (int k=1; k<=dim; k++)
       tmp = tmp*Width((*this)[k]);
       return tmp;
} */
//*********************************************************************
//******************    FONCTION NON MEMBRES     **********************
//*********************************************************************
int AxePrincipal(box& x)
	{	double widthmax=Width(x[1]);
		int kmax=1;
		for (int k=2; k<=x.dim; k++)
		{	if (Width(x[k])>widthmax)
			{ kmax=k;widthmax=Width(x[k]);};
		}
		return kmax;
	}

int AxePrincipal(box& x, box& V)
{ int k=1;
  interval I=x[k];
  double widthmax=Width(I)*fabs(Inf(V[k]));
  int kmax=k;
  for (k=2; k<=x.dim; k++)
  {	if (Width(x[k])*fabs(Inf(V[k]))>widthmax)
  	{ kmax=k;widthmax=Width(x[k])*fabs(Inf(V[k]));};
  	//if (w<0) AfxMessageBox ("Erreur axe principal1");
  }
  return kmax;
}
//----------------------------------------------------------------------
void Bisect(box& x, box& x1, box& x2)
{	BisectAlong(x,x1,x2,AxePrincipal(x));
}
//----------------------------------------------------------------------
void BisectAlong(box& x, box& x1, box& x2, int i)
{	x1 = x;	x2 = x;
	double mx = Center(x[i]);
	(x1[i]).SetSup(mx);
	(x2[i]).SetInf(mx);
        x1.pere=x.number;
        x2.pere=x.number;
        x1.number=-1;         x2.number=-1;
        x1.deja_evalue=false;      x2.deja_evalue=false;
        x1.nombre_ascendants=x.nombre_ascendants+1;
        x2.nombre_ascendants=x.nombre_ascendants+1;
}
//----------------------------------------------------------------------
box Center(const box& v)
	{   interval I;
		box r(v.dim);
		for (int k=1; k<=r.dim; k++) {I=v[k];	r[k]=Center(I);}
		return (r);
	}
//----------------------------------------------------------------------
bool Disjoint (const box& a,const box& b)
{   for (int i=1;i<=a.dim;i++)
    { if (Disjoint(a[i],b[i])) return true; }
    return false;
}
//----------------------------------------------------------------------
iboolean In(box& x,box& y)
	{
		if (x.isEmpty) return itrue;
		if (y.isEmpty) return ifalse;
	    iboolean r=itrue;
		for (int k=1; k<=x.dim; k++)
		{	iboolean in1=In(x[k],y[k]);
			if		(in1==false)   return ifalse;
			else if (in1.value==iperhaps) r=iperhaps;
		}
		return r;
	}
//----------------------------------------------------------------------
box Inf(box& v)
{   interval I;
	box r(v.dim);
	for (int k=1; k<=r.dim; k++) {I=v[k];	r[k]=Inf(I);}
	return (r);
}
//----------------------------------------------------------------------
box Inter(const box& a,const box& b)
{   box r(a.dim);
    if ((a.isEmpty)||(b.isEmpty))
        {r.isEmpty=true;
        return r;};
    for (int k=1; k<=r.dim; k++)
           {    r[k]=Inter(a[k],b[k]);
                if (r[k].isEmpty) {r.isEmpty=true;return r;}
           }
	return (r);
}
//----------------------------------------------------------------------
box Inflate(box& a, double eps)
{   box r(a.dim);
    r.isEmpty=a.isEmpty;
    for (int k=1; k<=r.dim; k++)
           {    r[k]=Inflate(a[k],eps);}
	return (r);
}
//----------------------------------------------------------------------
bool Prop(box& x,box& y)  // Normalement x is a subset of y (used in SIVEX)
	{   if (x.isEmpty) return false;
		if (y.isEmpty) return false;
	    for (int k=1; k<=x.dim; k++)
            if (Inf(x[k])==Inf(y[k]) ||  (Sup(x[k])==Sup(y[k])) ) return (true);
        return false;
	}
//----------------------------------------------------------------------
box Sup(box& v)
{   interval I;
	box r(v.dim);
	for (int k=1; k<=r.dim; k++) {I=v[k];	r[k]=Sup(I);}
	return (r);
}
//----------------------------------------------------------------------
// Face renvoie la boite qui est la face de la boite v
box Face(box& v, vector<iboolean> vector)
{   interval I;
	box r(v.dim);
	for (int k=1; k<=r.dim; k++)
        {I=v[k];
        if (vector[k-1]==itrue) v[k]=Sup(I);
        else if (vector[k-1]==iperhaps) v[k]=I;
        else v[k]=Inf(I);
        }
	return (r);
};
//----------------------------------------------------------------------
// renvoie la plus petite boite contenant la reunion des deux boites
box Reunion(box& a, box& b)
{   box r(a.dim);
    if (a.isEmpty) return (b);
    if (b.isEmpty) return (a);
	for (int k=1; k<=r.dim; k++)
           {    r[k]=Union(a[k],b[k]);
                if (r[k].isEmpty) r.isEmpty=true;}
	return (r);
}
box Union(box& a, box& b)
{   box r(a.dim);
    if (a.isEmpty) return (b);
    if (b.isEmpty) return (a);
	for (int k=1; k<=r.dim; k++)
           {    r[k]=Union(a[k],b[k]);
                if (r[k].isEmpty) r.isEmpty=true;}
	return (r);
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------
// renvoie la boite plac au centre de gravit des boites ponctuelles contenu dans vect
box centre_gravite (vector <box> vect)
{
//        box r(vect[0].dim);
        box r(2);
        double nombre;
        int alpha = vect.size();
	for (int k=1; k<=r.dim; k++)
           {
           nombre = 0;
           for (int j=0; j<vect.size(); j++)
           {nombre=Inf((vect[j])[k])+nombre;}
           nombre = nombre / vect.size();
           r[k] = interval(nombre,nombre);
           }
        return (r);
}
//----------------------------------------------------------------------
bool Subset(box& x,box& y)
{	if (x.isEmpty) return true;
 	if (y.isEmpty) return false;
    bool b=true;
 	for (int k=1; k<=x.dim; k++) b = b && Subset(x[k],y[k]);
 	return (b);
}
//----------------------------------------------------------------------
bool Subset(box& x,box& y,double epsilon)
{	if (Subset(x,y)==false) return false;
    for (int k=1; k<=x.dim; k++)
       if (Subset(x[k],y[k],epsilon)) return true;
    return (false);
}
//----------------------------------------------------------------------
/*void Sucre(box& P, box& S)    //Sucre le bout de P qui est dans S
{   int sum=0;
    int j=1;
    for (int i=1;i<=P.dim;i++)
         {if (Subset(P[i],S[i])) sum++; else j=i;}
    if (sum==(P.dim)-1)
       if (!Subset(S[j],P[j])) Sucre(P[j],S[j]);
}; */
//----------------------------------------------------------------------
double Volume(box& x)
{   double vol=1;
    for (int i=1;i<=x.dim;i++) vol=vol*Width(x[i]);
    return vol;
};
//----------------------------------------------------------------------
double Width(box& x)	{return Width(x[AxePrincipal(x)]);	}
//----------------------------------------------------------------------
double Width(box& x,box& V)
	{	int i=AxePrincipal(x,V);
        return (Width(x[i])*fabs(Inf(V[i])));	}
//----------------------------------------------------------------------
interval ProduitScalaire(box& u,box& v)
{  interval somme=0;
   for (int i=1; i<=u.dim; i++)  somme=somme+u[i]*v[i];
   return (somme);
}
//----------------------------------------------------------------------
interval Determinant(box& u,box& v)
{   interval u1=u[1];
    interval v2=v[2];
    interval v1=v[1];
    interval u2=u[2];
    interval r=u1*v2-v1*u2;
    return u1*v2-v1*u2; }
//----------------------------------------------------------------------
// Calcule l'angle entre 2 vecteur de dimension 2
double Angle (box& v, box& w)
// Attention, il faut des vecteurs et non des box et les vecteurs doivent etre
// de dimension 2
{   if ((v.dim!=2)||(w.dim!=2))
    {
       cout<<"error";
    }
    double n2,costeta,sinteta;
    interval nv=Norm(v);
    interval nw=Norm(w);
    interval nvw=nv*nw;
    n2=ToReel(nvw);
    costeta=ToReel(ProduitScalaire(v,w)/n2);
    sinteta=ToReel(Determinant(v,w))/n2;
    return(Arccossin(costeta,sinteta));
}

interval Norm(box& x)
{ int i;
  interval r=0;
  for (i=1; i<=x.dim; i++) r=r+Sqr(x[i]);
  return(Sqrt(r));
}










